@@ -3,44 +3,91 @@ use strict; use warnings;
 use Salus;
 use Rope;
 use Rope::Autoload;
-use feature qw/say/;
 use Term::Table;
 use Term::Size;
 use JSON qw//;
+use ANSI::Palette qw/bold_16 reset/;
 
 prototyped (
 	info => [qw/
 		title help_title header_title header_description read_title read_desc 
 		add_title add_desc all_title all_desc get_title get_desc get_col_title 
 		get_col_desc set_title set_desc set_col_title set_col_desc delete_title 
-		delete_desc delete_col_title delete_col_desc write_title write_desc exit_title exit_desc/
-	],
+		delete_desc delete_col_title delete_col_desc write_title write_desc 
+		sort_title sort_desc search_title search_desc exit_title exit_desc
+	/],
 	title => qq|Welcome to the command line interface for Salus\n|, 
 	help_title => qq|The following are commands that can be used to manipulate a Salus CSV\n|,
 	header_title => q|+ Headers|,
-	header_description => qq|\tprint the headers of the csv - headers|,
+	header_description => [
+		qq|\tprint the headers of the csv - |,
+		qq|headers|,
+	],
 	read_title => qq|+ Read File|,
-	read_desc => qq|\tread a salus csv file - read \$filepath|,
+	read_desc => [
+		qq|\tread a salus csv file -|,
+		qq|read \$filepath|,
+	],
 	add_title => qq|+ Add Row|,
-	add_desc => qq|\tadd a new row, the delimeter is a space not wrapped in quotations - add \$one \$two \$three|,
+	add_desc => [
+		qq|\tadd a new row, the delimeter is a space not wrapped in quotations - |,
+		qq|add \$one \$two \$three|,
+	],
 	all_title => qq|+ All|,
-	all_desc => qq|\tprint all rows - all|,
+	all_desc => [
+		qq|\tprint all rows - |,
+		qq|all|
+	],
 	get_title => qq|+ Get Row|,
-	get_desc => qq|\tretrieve a row by index and print it to the terminal - get \$index|,
+	get_desc => [
+		qq|\tretrieve a row by index and print it to the terminal - |,
+		qq|get \$index|,
+	],
 	get_col_title => qq|+ Get Row Column|,
-	get_col_desc => qq|\tretrieve a column by row and column index and print it to the terminal - get_col \$row \$col|,
+	get_col_desc => [
+		qq|\tretrieve a column by row and column index and print it to the terminal - |,
+		qq|get_col \$row \$col|
+	],
 	set_title => qq|+ Set Row|,
-	set_desc => qq|\tset an existing row by index - set \$index|,
+	set_desc => [
+		qq|\tset an existing row by index - |,
+		q|set \$index|,
+	],
 	set_col_title => qq|+ Set Row Column|,
-	set_col_desc => qq|\tset a columns value by row and column index - set_col \$row \$col|,
+	set_col_desc => [
+		qq|\tset a columns value by row and column index - |,
+		qq|set_col \$row \$col|,
+	],
 	delete_title => qq|+ Delete Row|,
-	delete_desc => qq|\tdelete a row by index - delete \$row|,
+	delete_desc => [
+		qq|\tdelete a row by index - |,
+		qq|delete \$row|
+	],
 	delete_col_title => qq|+ Delete Row|,
-	delete_col_desc => qq|\tdelete a column by row index and column index - delete \$row \$col|,
+	delete_col_desc => [
+		qq|\tdelete a column by row index and column index - |,
+		qq|delete \$row \$col|
+	],
 	write_title => qq|+ Write File|,
-	write_desc => qq|\twrite data to a file - write \$filepath|,
+	write_desc => [
+		qq|\twrite data to a file - |,
+		qq|write \$filepath|
+	],
 	exit_title => qq|+ Exit|,
-	exit_desc => qq|\tto exit you can either use - CTRL C -  or type - exit|,
+	exit_desc => [
+		qq|\tto exit you can either use - CTRL C - or type - |,
+		qq|exit|,
+	],
+	sort_title => qq|+ Sort|,
+	sort_desc => [
+		qq|\treorder the rows by column index - |,
+		qq|sort \$column \$dir|
+	],
+	search_title => qq|+ Search|,
+	search_desc => [
+		qq|\tsearch all rows by column value and print any matches to the terminal - |,
+		qq|search \$column \$search|
+	],
 	salus => undef
 );
 
@@ -56,7 +103,7 @@ function INITIALISED => sub {
 		$self->class = $class = $class->new();
 	} else {
 		if (!$params->{headers}) {
-			say "No class or headers passed while starting the salus script";
+			self->say("No class or headers passed while starting the salus script");
 			exit;
 		}
 
@@ -76,13 +123,11 @@ function run => sub {
 	my ($self, $params) = @_;
 
 	for (@{ $self->info }) {
-		say $self->$_;
+		$self->say($self->$_);
 	}
 
-	my $switch = $self->routes();
-
 	while (1) {
-		print "cmd: ";
+		$self->say("cmd: ", 1);
 		my $input = <STDIN>;
 		chomp($input);
 		if ($input eq 'exit') {
@@ -123,7 +168,7 @@ function get => sub {
 	my ($self, $index) = @_;
 	my $row = $self->salus->get_row($index);
 	unless ($row) {
-		say "No row available for index ${index}";
+		$self->say("No row available for index ${index}");
 	 	return;
 	}
 	$self->print_table($row->as_array);
@@ -173,6 +218,21 @@ function headers => sub {
 	$self->print_table();
 };
 
+function sort => sub {
+	my ($self, $col, $order) = @_;
+	$self->salus->sort($col, $order);
+};
+
+function search => sub {
+	my ($self, $col, $search) = @_;
+	my $rows = [
+		map {
+			$_->as_array;
+		} @{ $self->salus->search($col, $search) }
+	];
+	$self->print_table($rows);
+};
+
 function print_table => sub {
 	my ($self, $rows) = @_;
 	my ($columns, $r) = Term::Size::chars *STDOUT{IO};
@@ -192,7 +252,42 @@ function print_table => sub {
 			]
 		))
 	);
-	say $_ for $table->render;
+	print "$_\n" for $table->render;
+};
+
+function say => sub {
+	my ($self, $string, $no, $colour) = @_;
+	
+	if (ref $string) {
+		$self->say($string->[0], 1);
+		$self->say($string->[1], 0, 1);
+		print "\n";
+		return;
+	}
+	
+	my %colours;
+	%colours = (
+		colour => sub { return "\e[$_[0];1;1m" . $_[1]; },
+		custom => sub { $colours{colour}->(@_) . "\e[0m" . $colours{base}->(''); },
+		header => sub { $colours{colour}->(@_) . "\e[0m" . $colours{colour}->('34', ''); },
+		'-' => sub { $colours{custom}->(31, @_); },
+		'+' => sub { $colours{header}->(32, @_); },
+		'$' => sub { $colours{custom}->(35, @_); },
+		'key' => sub { $colours{custom}->(36, @_); },
+		'cmd:' => sub { $colours{custom}->(32, @_); },
+		'base' => sub { $colours{colour}->(33, @_) },
+	);
+	
+	$string =~ s/(-|\+|cmd\:)/$colours{$1}->($1)/eg;
+
+	if ($colour) {
+		$string =~ s/((\$)[^\s]+|[^\s]+)/$colours{$2||'key'}->($1)/eg;
+	} else {
+		$string = $colours{base}->($string);
+	}
+
+	print $string;
+	print "\e[0m\n" unless $no;
 };
 
 1;
